//DARM.java

import java.io.*;
import java.lang.*;
import java.util.*;

////////////////////DARM class
public class DARM
{
	Site[] Sites;
	int nSites;
	
	ItemsetCollection DB;
	ItemsetCollection L;
	double supportThreshold;
	double confidenceThreshold;
	AssociationRuleCollection Rules;
	
	int nTotalSentMessages=0;
	int nTotalReceivedMessages=0;
	
	//constructor
	public DARM()
	{
		DB=new ItemsetCollection();
		L=new ItemsetCollection();
		supportThreshold=0.0;
		confidenceThreshold=0.0;

		int maxSites=10;
		Sites=new Site[maxSites];
		for(int t=0;t<maxSites;t++) Sites[t]=new Site();
		nSites=0;
		
		Rules=new AssociationRuleCollection();
	}
	
	//get functions
	public int get_nSites()
	{
		return(nSites);
	}
	
	public Site getSite(int tIndex)
	{
		Site tSite=new Site();
		
		if(nSites>0)
		{
			if(tIndex>=0&&tIndex<=nSites-1)
			{
				tSite=Sites[tIndex];				
			}
		}
		
		return(tSite);
	}
	
	public ItemsetCollection getL()
	{
		return(L);
	}

	public double getSupportThreshold()
	{
		return(supportThreshold);
	}

	public double getConfidenceThreshold()
	{
		return(confidenceThreshold);
	}

	public double get_nRules()
	{
		return(Rules.get_nAssociationRules());
	}

	public AssociationRule getRule(int tIndex)
	{
		AssociationRule tRule=new AssociationRule();

		if(Rules.get_nAssociationRules()>0)
		{
			if(tIndex>=0&&tIndex<=Rules.get_nAssociationRules()-1)
			{
				tRule=Rules.getAssociationRule(tIndex);
			}
		}

		return(tRule);
	}

	//set functions
	public void setL(ItemsetCollection tItemsetCollection)
	{
		L.clear();
		L=tItemsetCollection;
	}

	public void setSupportThreshold(double tSupport)
	{
		supportThreshold=tSupport;
	}

	public void setConfidenceThreshold(double tConfidence)
	{
		confidenceThreshold=tConfidence;
	}

	//methods
	public void addSite(Site tSite)
	{
		Sites[nSites]=tSite;
		nSites++;
	}
	
	public void addSite(String tSiteName,String tDBFilePath)
	{
		Site tSite=new Site();
		tSite.setSiteName(tSiteName);
		tSite.setDBFilePath(tDBFilePath);
		tSite.read();
		tSite.findI();
		tSite.setSupportThreshold(supportThreshold);
		tSite.setConfidenceThreshold(confidenceThreshold);
		addSite(tSite);
	}
	
	public void AddRule(AssociationRule tRule)
	{
		Rules.addAssociationRule(tRule);
	}

	//odam algorithm
	public void GenerateL_ODAM()
	{
		//find globalSupportCountThreshold
		int tDBSize=0;
		System.out.println("LocalSupportCountThreshold:");
		for(int t=0;t<nSites;t++)
		{
			System.out.println(Sites[t].getSiteName()+": "+Sites[t].getSupportCountThreshold());
			tDBSize+=Sites[t].getDB().get_nItemsets();
		}

		double globalSupportThreshold=supportThreshold;
		int globalSupportCountThreshold=(int)((globalSupportThreshold/100.0)*tDBSize);
		System.out.println("\nGlobalSupportCountThreshold: "+globalSupportCountThreshold+"\n");
		
		int k=1;
		boolean flag=true;
		while(flag==true)
		{
			for(int t=0;t<nSites;t++)
			{
				//receive from sender
				ItemsetCollection ic1=new ItemsetCollection();
				ic1=Sites[t].GenerateL(k);
				nTotalReceivedMessages+=ic1.get_nItemsets();
				
				//check globalSupportCountThreshold
				ItemsetCollection ic2=new ItemsetCollection();
				for(int j=0;j<=ic1.get_nItemsets()-1;j++)
				{
					//get lastitem as local support
					String tItem=ic1.getItemset(j).getItem(ic1.getItemset(j).get_nItems()-1);
					ic1.getItemset(j).removeItem(tItem);
					if(ic1.getItemset(j).get_nItems()>0)
					{
						double tSupportCount=Integer.parseInt(tItem);
						if(tSupportCount>=globalSupportCountThreshold)
						{
							ic2.addItemset(ic1.getItemset(j));
						}
					}
				}
				
				//send back to receiver
				Sites[t].SetLi(ic2);
				nTotalSentMessages+=ic2.get_nItemsets();

				if(ic1.get_nItemsets()==0)
				{
					flag=false;
					break;
				}
				L.appendItemsetCollection(ic2);
			}
			
			k=k+1;
			//break;
		}
		ItemsetCollection ict=new ItemsetCollection();
		ict=L.getUniqueItemsetCollection();
		L=ict;

		System.out.println("Statistics:\n");
		for(int t=0;t<nSites;t++)
		{
			System.out.println(Sites[t].getSiteName()+":");
			System.out.println("Sent: "+Sites[t].get_nSentMessages());
			System.out.println("Received: "+Sites[t].get_nReceivedMessages());
			System.out.println();
		}

		System.out.println("Total:");
		System.out.println("Total Sent: "+nTotalSentMessages);
		System.out.println("Total Received: "+nTotalReceivedMessages);
		System.out.println();
	}

	public void Mine()
	{
		Itemset tLItemset=new Itemset();
		Itemset tXItemset=new Itemset();
		ItemsetCollection tLSubSets=new ItemsetCollection();
		Itemset tItemset1=new Itemset();
		AssociationRule tRule=new AssociationRule();
		double tConfidence=0;
		int i,j;

		Rules.clear();
		for(i=0;i<=L.get_nItemsets()-1;i++)
		{
			tLItemset.setItems(L.getItemset(i));
			tLSubSets=tLItemset.GetSubSets(0);
			for(j=0;j<=tLSubSets.get_nItemsets()-1;j++)
			{
				tXItemset.setItems(tLSubSets.getItemset(j));

				tConfidence=(DB.getSupport(tLItemset)/DB.getSupport(tXItemset))*100;
				if(tConfidence>=confidenceThreshold)
				{
					//add rule
					//set x
					tRule.setX(tXItemset);
					//set y
					tItemset1.setItems(tLItemset);
					tItemset1.removeItemset(tXItemset);
					tRule.setY(tItemset1);
					//set support and confidence
					tRule.setSupport(DB.getSupport(tLItemset));
					tRule.setConfidence(tConfidence);

					if(tRule.getX().get_nItems()>0&&tRule.getY().get_nItems()>0)
					{
						AddRule(tRule);
					}
				}
			}
		}
	}
	
	public String toString()
	{
		//support&confidence
		String tStrSC;
		tStrSC="Support Threshold: "+supportThreshold+"\n\r\n\r";
		tStrSC=tStrSC+"Confidence Threshold: "+confidenceThreshold+"\n\r"+"\n\r\n\r";

		//L
		String tStrL;
		tStrL="Large Itemset (L): "+L.get_nItemsets()+" itemset(s)"+"\n\r\n\r"+L.toString()+"\n\r";

		//rules
		String tStrR="";
		tStrR="Association Rules: "+Rules.get_nAssociationRules()+" rule(s)"+"\n\r\n\r"+Rules.toString1();

		String tStr=tStrSC+tStrL+tStrR;
		return(tStr);
	}

	public String toString1()
	{
		//support&confidence
		String tStrSC;
		tStrSC="Support Threshold: "+supportThreshold+"\n";
		tStrSC=tStrSC+"Confidence Threshold: "+confidenceThreshold+"\n"+"\n";

		//L
		String tStrL;
		tStrL="Large Itemset (L): "+L.get_nItemsets()+" itemset(s)"+"\n"+L.toString1()+"\n";

		//rules
		String tStrR="";
		tStrR="Association Rules: "+Rules.get_nAssociationRules()+" rule(s)"+"\n"+Rules.toString1();

		String tStr=tStrSC+tStrL+tStrR;
		return(tStr);
	}
}
