//Site.java

import java.io.*;
import java.lang.*;
import java.util.*;

////////////////////Site class
public class Site
{
	String SiteName;
	String DBFilePath;
	
	ItemsetCollection DB;
	Itemset I;
	ItemsetCollection L;
	double supportThreshold;
	int supportCountThreshold;
	double confidenceThreshold;
	AssociationRuleCollection Rules;

	ItemsetCollection Ci=new ItemsetCollection();
	ItemsetCollection Li=new ItemsetCollection();
	int nSentMessages=0;
	int nReceivedMessages=0;

	//constructor
	public Site()
	{
		SiteName="";
		DBFilePath="";
		
		DB=new ItemsetCollection();
		I=new Itemset();
		L=new ItemsetCollection();
		supportThreshold=0.0;
		supportCountThreshold=0;
		confidenceThreshold=0.0;
		
		Rules=new AssociationRuleCollection();
	}

	//get functions
	public String getSiteName()
	{
		return(SiteName);
	}
	
	public String getDBFilePath()
	{
		return(DBFilePath);
	}
	
	public ItemsetCollection getDB()
	{
		return(DB);
	}

	public Itemset getI()
	{
		return(I);
	}

	public ItemsetCollection getL()
	{
		return(L);
	}

	public double getSupportThreshold()
	{
		return(supportThreshold);
	}
	
	public int getSupportCountThreshold()
	{
		return(supportCountThreshold);
	}

	public double getConfidenceThreshold()
	{
		return(confidenceThreshold);
	}

	public double get_nRules()
	{
		return(Rules.get_nAssociationRules());
	}

	public AssociationRule getRule(int tIndex)
	{
		AssociationRule tRule=new AssociationRule();

		if(Rules.get_nAssociationRules()>0)
		{
			if(tIndex>=0&&tIndex<=Rules.get_nAssociationRules()-1)
			{
				tRule=Rules.getAssociationRule(tIndex);
			}
		}

		return(tRule);
	}
	
	public int get_nSentMessages()
	{
		return(nSentMessages);
	}
	
	public int get_nReceivedMessages()
	{
		return(nReceivedMessages);
	}

	//set functions
	public void setSiteName(String tSiteName)
	{
		SiteName=tSiteName;
	}
	
	public void setDBFilePath(String tDBFilePath)
	{
		DBFilePath=tDBFilePath;
	}
	
	public void setDB(ItemsetCollection tItemsetCollection)
	{
		DB.clear();
		DB=tItemsetCollection;
	}

	public void setL(ItemsetCollection tItemsetCollection)
	{
		L.clear();
		L=tItemsetCollection;
	}

	public void setSupportThreshold(double tSupport)
	{
		supportThreshold=tSupport;
		int tSupportCountThreshold=(int)((tSupport/100.0)*DB.get_nItemsets());
		setSupportCountThreshold(tSupportCountThreshold);
	}
	
	public void setSupportCountThreshold(int tSupportCount)
	{
		supportCountThreshold=tSupportCount;
	}

	public void setConfidenceThreshold(double tConfidence)
	{
		confidenceThreshold=tConfidence;
	}

	//methods
	public void read()
	{
		try
		{
			FileInputStream fin;
			fin=new FileInputStream(DBFilePath);
			int ch=0;
			Itemset tItemset=new Itemset();
			ItemsetCollection tItemsetCollection=new ItemsetCollection();
			
			String tmp="";
			while((ch=fin.read())!=-1)
			{
				if(ch==32)
				{
					tItemset.addItem(tmp);
					tmp="";
					continue;
				}
				if(ch==13)
				{
					tItemset.addItem(tmp);
					tItemsetCollection.addItemset(tItemset);
					tItemset.clear();
					tmp="";
					fin.read();
					continue;
				}
				tmp+=(char)ch;
			}
        
			setDB(tItemsetCollection);
		}
		catch(Exception e)
		{
			System.out.println("Error: "+e.getMessage());
		}
	}

	public void AddRule(AssociationRule tRule)
	{
		Rules.addAssociationRule(tRule);
	}

	public void findI()
	{
		I.clear();
		I.setItems(DB.getUniqueItemset());
	}

	public void GenerateL_Apriori()
	{
		int k,t,i1,j1;
		ItemsetCollection Ci=new ItemsetCollection();
		ItemsetCollection Li=new ItemsetCollection();
		Itemset tItemset=new Itemset();

		k=1;
		L.clear();

		//set C1 as I
		Ci.clear();
		for(t=0;t<=I.get_nItems()-1;t++)
		{
			Ci.addItemset(new Itemset(I.getItem(t)));
		}

		while(Ci.get_nItemsets()!=0)
		{
			//set Li from Ci
			Li.clear();
			for(t=0;t<=Ci.get_nItemsets()-1;t++)
			{
				if(DB.getSupportCount(Ci.getItemset(t))>=supportCountThreshold)
				{
					Li.addItemset(Ci.getItemset(t));
					L.addItemset(Ci.getItemset(t));
				}
			}

			//find supersets of Li to Ci
			Ci.clear();
			Itemset tUItemset=new Itemset();
			tUItemset=Li.getUniqueItemset();
			Ci=tUItemset.GetSubSets(k+1);
			k=k+1;
		}
	}

	//message sending
	public ItemsetCollection GenerateL(int k)
	{
		//System.out.println("["+SiteName+"]");
		
		//genereate Ci
		Ci.clear();
		if(k==1)
		{
			L.clear();
			for(int t=0;t<=I.get_nItems()-1;t++)
			{
				Ci.addItemset(new Itemset(I.getItem(t)));
			}
		}
		else
		{
			Itemset tUItemset=new Itemset();
			tUItemset=Li.getUniqueItemset();
			Ci=tUItemset.GetSubSets(k);
		}
		//System.out.println(Ci.toString());

		//generate Li from Ci
		Li.clear();
		for(int t=0;t<=Ci.get_nItemsets()-1;t++)
		{
			if(DB.getSupportCount(Ci.getItemset(t))>=supportCountThreshold)
			{
				Ci.getItemset(t).addItem(String.valueOf(DB.getSupportCount(Ci.getItemset(t))));
				//if(Ci.getItemset(t).get_nItems()>1)
				//{
					Li.addItemset(Ci.getItemset(t));
					L.addItemset(Ci.getItemset(t));
				//}
			}
		}
		nSentMessages+=Li.get_nItemsets();
		
		if(Li.get_nItemsets()>1)
		{
			System.out.println(SiteName+":");
			System.out.println("Sending "+Li.get_nItemsets()+" messages:");
			System.out.println(Li.toString1());
		}
		return(Li);
	}
	
	//message receiving
	public void SetLi(ItemsetCollection tItemsetCollection)
	{
		Li=tItemsetCollection;
		nReceivedMessages+=Li.get_nItemsets();

		if(Li.get_nItemsets()>1)
		{
			System.out.println(SiteName+":");
			System.out.println("Receiving "+Li.get_nItemsets()+" messages:");
			System.out.println(Li.toString1()+"\n");
		}
	}

	public void Mine()
	{
		Itemset tLItemset=new Itemset();
		Itemset tXItemset=new Itemset();
		ItemsetCollection tLSubSets=new ItemsetCollection();
		Itemset tItemset1=new Itemset();
		AssociationRule tRule=new AssociationRule();
		double tConfidence;
		int i,j;

		Rules.clear();
		for(i=0;i<=L.get_nItemsets()-1;i++)
		{
			tLItemset.setItems(L.getItemset(i));
			tLSubSets=tLItemset.GetSubSets(0);
			for(j=0;j<=tLSubSets.get_nItemsets()-1;j++)
			{
				tXItemset.setItems(tLSubSets.getItemset(j));

				tConfidence=(DB.getSupport(tLItemset)/DB.getSupport(tXItemset))*100;
				if(tConfidence>=confidenceThreshold)
				{
					//add rule
					//set x
					tRule.setX(tXItemset);
					//set y
					tItemset1.setItems(tLItemset);
					tItemset1.removeItemset(tXItemset);
					tRule.setY(tItemset1);
					//set support and confidence
					tRule.setSupport(DB.getSupport(tLItemset));
					tRule.setConfidence(tConfidence);

					if(tRule.getX().get_nItems()>0&&tRule.getY().get_nItems()>0)
					{
						AddRule(tRule);
					}
				}
			}
		}
	}
	
	public String toString()
	{
		//dbinfo
		String tStrDB;
		tStrDB="DB: "+DB.get_nItemsets()+" transactions"+"\n\r\n\r"+"\n\r";

		//I
		String tStrI;
		tStrI="Unique Itemset (I): "+I.get_nItems()+" item(s)"+"\n\r\n\r"+I.toString()+"\n\r"+"\n\r\n\r";

		//support&confidence
		String tStrSC;
		tStrSC="SupportCount Threshold: "+supportCountThreshold+"\n\r\n\r";
		tStrSC=tStrSC+"Confidence Threshold: "+confidenceThreshold+"\n\r"+"\n\r\n\r";

		//L
		String tStrL;
		tStrL="Large Itemset (L): "+L.get_nItemsets()+" itemset(s)"+"\n\r\n\r"+L.toString()+"\n\r";

		//rules
		String tStrR="";
		tStrR="Association Rules: "+Rules.get_nAssociationRules()+" rule(s)"+"\n\r\n\r"+Rules.toString1();

		String tStr=SiteName+":\n\r\n\r\n\r"+tStrDB+tStrI+tStrSC+tStrL+tStrR;
		return(tStr);
	}
}
